Package: bumphunter
Version: 1.55.1
Title: Bump Hunter
Description: Tools for finding bumps in genomic data
Authors@R: c(person(c("Rafael", "A."), "Irizarry", role = "aut", email
    = "rafa@ds.dfci.harvard.edu"), person("Martin", "Aryee", role = "aut"),
    person(c("Kasper", "Daniel"), "Hansen", role = "aut"), person(c("Hector",
    "Corrada"), "Bravo", role = "aut"), person("Shan", "Andrews", role =
    "ctb"), person(c("Andrew", "E."), "Jaffe", role = "ctb"), person("Harris",
    "Jaffee", role = "ctb"), person("Leonardo", "Collado-Torres", role="ctb"),
    person("Tamilselvi", "Guharaj", role = "cre", email =
    "selvi@ds.dfci.harvard.edu"))
Depends: R (>= 3.5), S4Vectors (>= 0.9.25), IRanges (>= 2.3.23),
        Seqinfo, GenomicRanges, foreach, iterators, methods, parallel,
        locfit
Suggests: testthat, RUnit, doParallel, GenomeInfoDb, txdbmaker,
        org.Hs.eg.db, TxDb.Hsapiens.UCSC.hg19.knownGene
Imports: matrixStats, limma, doRNG, BiocGenerics, utils,
        GenomicFeatures, AnnotationDbi, stats
License: Artistic-2.0
LazyData: yes
URL: https://github.com/rafalab/bumphunter
biocViews: DNAMethylation, Epigenetics, Infrastructure,
        MultipleComparison, ImmunoOncology
RoxygenNote: 6.1.1
Config/pak/sysreqs: make libbz2-dev liblzma-dev libpng-dev libxml2-dev
        libssl-dev xz-utils zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-05-25 21:15:24 UTC
RemoteUrl: https://github.com/bioc/bumphunter
RemoteRef: HEAD
RemoteSha: 51b589f2c3be64859df7b50431005b8cec3b72d7
NeedsCompilation: no
Packaged: 2026-05-26 07:47:51 UTC; root
Author: Rafael A. Irizarry [aut],
  Martin Aryee [aut],
  Kasper Daniel Hansen [aut],
  Hector Corrada Bravo [aut],
  Shan Andrews [ctb],
  Andrew E. Jaffe [ctb],
  Harris Jaffee [ctb],
  Leonardo Collado-Torres [ctb],
  Tamilselvi Guharaj [cre]
Maintainer: Tamilselvi Guharaj <selvi@ds.dfci.harvard.edu>
Built: R 4.6.0; ; 2026-05-26 07:53:08 UTC; unix
