\docType{data}
\name{TT}
\alias{TT}
\title{Example data}
\format{A list with three components.
\describe{
\item{txdb }{  Has a TxDb example.}
\item{org }{ has an Org DB example.}
\item{transcripts }{ has example transcripts output.}
}}
\description{
Example data
}
\keyword{datasets}
